#include <sourcemod>
#include <unixtime_sourcemod>
#include <clientprefs>
#include <cstrike>

#pragma semicolon 1
#pragma newdecls required

bool g_bCookiePrefs[MAXPLAYERS + 1];  

Handle g_hCookie = INVALID_HANDLE;

int iTimezone;

ConVar g_cSbTime;

public Plugin myinfo =
{
	name = "Chat-Timestamps",
	author = "Mug1wara",
	description = "Provides timestamps in chat, players with flag root get's to change the timezone which will be used to provide the time as prefix.",
	version = "1.0.2",
	url = "http://sourcemod.net/",
}

public void OnPluginStart()
{
	/* Similar chunk from Kyles clientprefs tutorial. */
	g_hCookie = RegClientCookie("OurTestCookie", "A Test Cookie for use in our Tutorial", CookieAccess_Private);
	SetCookiePrefabMenu(g_hCookie, CookieMenu_OnOff_Int, "");
	
	g_cSbTime = CreateConVar("Chat-Timestamps_Scoreboard", "1", "Sets whether the scoreboard time is enabled or disabled");
	
	RegAdminCmd("sm_cts", Cmd_ChatTimeStamps, ADMFLAG_ROOT);
	
	for (int i = 1; i <= MaxClients; i++)
	{
		if (!AreClientCookiesCached(i))
			continue;
        
		OnClientCookiesCached(i);
	}
	
	AutoExecConfig();
}

public void OnMapStart()
{
	CreateTimer(0.2, Timer_Tag, _, TIMER_REPEAT | TIMER_FLAG_NO_MAPCHANGE);
}

public Action Timer_Tag(Handle hTimer)
{
	char Sb[8];
	
	int iTime, iYear, iMonth, iDay, iHour, iMinute, iSecond;
	
	iTime = GetTime();
	
	UnixToTime(iTime, iYear, iMonth, iDay, iHour, iMinute, iSecond, iTimezone);
	
	if (g_cSbTime.IntValue == 0)
		return Plugin_Handled;
	
	for (int i = 1; i <= MaxClients; i++)
	{
		if (IsValidClient(i))
		{
			Format(Sb, sizeof(Sb), "%02d:%02d", iHour, iMinute);
			CS_SetClientClanTag(i, Sb);
		}
	}
	
	return Plugin_Continue;
}

public void OnClientCookiesCached(int iClient)
{
	char sValue[8];
	GetClientCookie(iClient, g_hCookie, sValue, sizeof(sValue));
	
	g_bCookiePrefs[iClient] = (sValue[0] != '\0' && StringToInt(sValue));
}  

public Action Cmd_ChatTimeStamps(int iClient, int sArgs)
{
	if (IsValidClient(iClient))
	{
		Menu hMenu = new Menu(Menu_Timestamp);
		hMenu.SetTitle("Select Your Timezone");
		hMenu.AddItem("", "Western Europe");
		hMenu.AddItem("", "Central Europe");
		hMenu.AddItem("", "Eastern Europe");
		hMenu.AddItem("", "Pacific Time");
		hMenu.AddItem("", "Central Time");
		hMenu.AddItem("", "Eastern Time");
		hMenu.Display(iClient, MENU_TIME_FOREVER);
	}
	
	return Plugin_Handled;
}

public int Menu_Timestamp(Menu hMenu, MenuAction hAction, int iClient, int iParam)
{
	switch (hAction)
	{
		case MenuAction_Select:
		{
			if (IsValidClient(iClient))
			{
				switch (iParam)
				{
					case 0: iTimezone = UT_TIMEZONE_WET;
					case 1: iTimezone = UT_TIMEZONE_CET;
					case 2: iTimezone = UT_TIMEZONE_EET;
					case 3: iTimezone = UT_TIMEZONE_PST;
					case 4: iTimezone = UT_TIMEZONE_CST;
					case 5: iTimezone = UT_TIMEZONE_EST;
				}
				
				OnClientCookiesCached(iClient);
			}
		}
		
		case MenuAction_End: delete hMenu;
	}
}

public Action OnClientSayCommand(int iClient, const char[] sCommand, const char[] sArgs)
{
	int iTime, iYear, iMonth, iDay, iHour, iMinute, iSecond;
	
	iTime = GetTime();
	
	UnixToTime(iTime, iYear, iMonth, iDay, iHour, iMinute, iSecond, iTimezone);
	
	if (StrEqual(sCommand, "say"))
	{
		int iTeam = GetClientTeam(iClient);
		
		if (iTeam > 1)
		{
			if (IsClientInGame(iClient))
			{
				if (GetClientTeam(iClient) == 3)
					PrintToChatAll("%02d:%02d - \x0B%N \x01» %s", iHour, iMinute, iClient, sArgs);
					
				else if (GetClientTeam(iClient) == 2)
					PrintToChatAll("%02d:%02d - \x0B%N \x01» %s", iHour, iMinute, iClient, sArgs);
			}
			
			return Plugin_Stop;
		}
	}
	
	else if (StrEqual(sCommand, "say_team"))
	{
		int iTeam = GetClientTeam(iClient);
		
		if (iTeam > 1)
		{
			for (int i = 1; i <= MaxClients; i++)
			{
				if (IsClientInGame(i))
				{
					if (iTeam == GetClientTeam(i))
					{
						if (iTeam == 3)
							PrintToChat(i, "%02d:%02d - \x0B(Counter-Terrorists) %N \x01» %s", iHour, iMinute, iClient, sArgs);
						
						else if (iTeam == 2)
							PrintToChat(i, "%02d:%02d - \x0B(Terrorists) %N \x01» %s", iHour, iMinute, iClient, sArgs);
					}
				}
			}
			
			return Plugin_Stop;
		}
	}
	
	return Plugin_Handled;
}

bool IsValidClient(int client)
{
	if (!(0 < client <= MaxClients) || !IsClientInGame(client) || IsFakeClient(client))
		return false;
	return true;
}